<?php

/**
 * @file
 * Integration with the loripsum.net content generation api for generating rich
 * content.
 *
 * Usage:
 *  Return 10 rows with a body field populated by the described options.
 *  paragraph size 
 *  @code
 *  $this->source = new MigrateGenerateSource(10, array(
 *    'body' => array(
 *      'type' => 'loripsum',
 *      'small',
 *      2, 
 *    ),
 *  ));
 *  @endcode
 *
 * @see http://loripsum.net
 */
class MigrateGenerateLoripsum extends MigrateGeneratePlugin {
  public function getFieldOutput($options, $field_name, $migration) {
    // Do all random preffered for a standard body field but no plaintext
    // and no headings
    if (!count($options)) {
      $options['random'] = array(
        range(1,8),
        array('small', 'medium', 'large'),
        'ul','ol','dl','bq','links',
        'decorate',
      );
    }
    // Handle random options first. 
    // Arrays randomly picked from, and others are randomly set or not set.
    if (isset($options['random'])) {
      foreach ($options['random'] AS $k => $v) {
        if (is_array($v)) {
          // This does rely on arrays having numerical indicies start at 0
          $options[] = $options['random'][$k][mt_rand(0, count($v)-1)];
        }
        elseif (mt_rand(0,1)) {
          $options[] = $v;
        }
      }
      // All options are passed to loripsum.net so we remove our own options
      unset($options['random']);
    }
    return file_get_contents('http://loripsum.net/api/' . implode('/', array_values($options)));
  }

  public function getFieldDescription($field_name, &$options) {
    return t('Rich content graciously generated by loripsum.net');
  }
}
