<?php

/**
 * @file
 * Migrate source handler for generating content.
 */

class MigrateGenerateSource extends MigrateSource {
  /**
   * Stores the field structure generaton rules.
   */
  protected $fields = array();


  protected $migration;
  protected $row_idx = 1;
  public function currentRowIDx() {
    return $this->row_idx;
  }

  public function __construct($num_rows, $migration, $fields) {
    $this->num_rows = $num_rows;
    $this->fields = $fields;
    $this->migration = $migration;
    parent::__construct();
    // Not sure if this is working
    $this->trackChanges = FALSE;
    //$this->needsUpdate = TRUE;
  }

  public function computeCount() {
    return $this->num_rows;
  }

  public function performRewind() {
    // May need to provide serials through this class so that they can 
    // be reset
    $this->row_idx = 1;
  }

  public function fields() {
    $return = array();
    foreach ($this->fields AS $field_name => $options) {
      if (!is_array($options)) {
        $options = array('plugin' => $options);
      }
      if (!empty($options['plugin'])) {
        $plugin = $options['plugin'];
        unset($options['plugin']);
        $options['field description'] = $plugin::getFieldDescription($field_name, $options, $this);
      }

      if (!empty($options['field description'])) {
        $return[$field_name] = $options['field description'];
      }
      else {
        $return[$field_name] = t('Generated by Migrate Generate');
      }
    }
    return $return;
  }

  public function __toString() {
    return t('Returns @num generated rows', array('@num' => $this->num_rows));
  }

  public function getNextRow() {
    if ($this->row_idx > $this->num_rows) {
      return NULL;
    }
    $row = array();
    foreach ($this->fields AS $field_name => $options) {
      if (!is_array($options)) {
        $param = $options;
        $options = array();
        if (is_callable($options)) {
          $options['callback'] = $param;
        }
        else {
          $options['plugin'] = $param;
        }
      }

      if (!empty($options['plugin'])) {
        $options['callback'] = "{$options['plugin']}::getFieldOutput";
        unset($options['plugin']);
      }

      if (!empty($options['callback'])) {
        $callback = $options['callback'];
        if (is_callable($options['callback']) || function_exists($options['callback'])) {
          // Populate our row with our callback (this is usually a static method
          // invocation of a class extending MigrateGeneratePlugin but any
          // callable callback is accepted
          unset($options['callback']);
          $row[$field_name] = call_user_func_array($callback, array($options, $field_name, $this->migration));
        }
        else {
          throw new Exception('Unknown callback "' . $callback . '"() in MigrateSourceGenerate');
        }
      }
      else {
        // there is a bug here
        throw new Exception('Unhandled field encountered in @class', array('@class' => get_class($this)));
      }
    }
    $row['csvrownum'] = $this->row_idx++;
    return (object)$row;
  }
}

/**
 * Use this class when your extending MigrationXML?
 */
class MigrateGenerateXMLSource extends MigrateGenerateSource {
  public function prepareRow(&$row) {
    // Provide the row as the xml for anything that expected it
    $row->xml = (object)$row;

    // Skip rows that don't satisfy prepareRow() by default
    return parent::prepareRow($row);
  }
}

/**
 * Static plugin handling to use autoloading of classes which allow other 
 * modules to easily define plugins with only having to register the file.
 *
 * As an attribute of options set plugin to the class name of the plugin you
 * wish to utilize. 
 * @code
 * $options['type'] = 'MyMigrateGeneratePlugin'
 * @endcode
 *
 * For more simple usage, use the "callback" option
 *
 * @code
 * $options['callback'] = 'my_custom_migrate_generate_callback';
 * @endcode
 *
 * All other options provided to the field are passed on to the plugin.
 *
 * @see MigrateGenerateSource::getNextRow()
 */
abstract class MigrateGeneratePlugin {
  abstract public function getFieldOutput($options, $field_name, $migration);

  public function getDefaultOptions() {
    return array();
  }

  /**
   * Return a description of this field for the migration.
   * 
   * Optionally override this to be more descriptive 
   * @param array $field_name
   *   This is the index of the field definition.
   * @param array $options
   *   Options may influence your description. 
   * @see MigrateGenerateSource::fields()
   */
  public function getFieldDescription($field_name, &$options) {
    // Allow overriding the field description in the options
    if (isset($options['field description'])) {
      $return = $options['field description'];
      unset($options['field description']);
      return $return;
    }
    else {
      // Return a generic description
      if (version_compare(phpversion(), '5.3', '<')) {
        return t('Generated by MigrateGenerate');
      }
      return t('Generated output of @plugin', array(
        '@plugin' => get_called_class(),
      ));
    }
  }
}
